"""
    Strawberry GraphQL Made Easy
"""
from .types import Edges as edges
from .types import Error as errors
from .types import ErrorMessage as error
from .types import Mutation as mutation
from .types import Page as page
from .types import Query as query
